@@ -1,12 +1,12 @@
-use ruff_diagnostics::{AlwaysFixableViolation, Diagnostic, Edit, Fix};
-use ruff_macros::{derive_message_formats, violation};
+use ruff_macros::{ViolationMetadata, derive_message_formats};
 use ruff_python_ast::visitor::Visitor;
-use ruff_python_ast::{self as ast, Expr, StmtFor};
+use ruff_python_ast::{self as ast, Expr, Int, Number, StmtFor};
 use ruff_python_semantic::SemanticModel;
 use ruff_text_size::Ranged;
 
 use crate::checkers::ast::Checker;
 use crate::rules::pylint::helpers::SequenceIndexVisitor;
+use crate::{AlwaysFixableViolation, Edit, Fix};
 
 /// ## What it does
 /// Checks for index-based list accesses during `enumerate` iterations.
@@ -31,22 +31,23 @@ use crate::rules::pylint::helpers::SequenceIndexVisitor;
 /// for index, letter in enumerate(letters):
 ///     print(letter)
 /// ```
-#[violation]
-pub struct UnnecessaryListIndexLookup;
+#[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "0.5.0")]
+pub(crate) struct UnnecessaryListIndexLookup;
 
 impl AlwaysFixableViolation for UnnecessaryListIndexLookup {
     #[derive_message_formats]
     fn message(&self) -> String {
-        format!("List index lookup in `enumerate()` loop")
+        "List index lookup in `enumerate()` loop".to_string()
     }
 
     fn fix_title(&self) -> String {
-        format!("Use the loop variable directly")
+        "Use the loop variable directly".to_string()
     }
 }
 
 /// PLR1736
-pub(crate) fn unnecessary_list_index_lookup(checker: &mut Checker, stmt_for: &StmtFor) {
+pub(crate) fn unnecessary_list_index_lookup(checker: &Checker, stmt_for: &StmtFor) {
     let Some((sequence, index_name, value_name)) =
         enumerate_items(&stmt_for.iter, &stmt_for.target, checker.semantic())
     else {
@@ -61,17 +62,16 @@ pub(crate) fn unnecessary_list_index_lookup(checker: &mut Checker, stmt_for: &St
     };
 
     for range in ranges {
-        let mut diagnostic = Diagnostic::new(UnnecessaryListIndexLookup, range);
+        let mut diagnostic = checker.report_diagnostic(UnnecessaryListIndexLookup, range);
         diagnostic.set_fix(Fix::safe_edits(
             Edit::range_replacement(value_name.id.to_string(), range),
             [noop(index_name), noop(value_name)],
         ));
-        checker.diagnostics.push(diagnostic);
     }
 }
 
 /// PLR1736
-pub(crate) fn unnecessary_list_index_lookup_comprehension(checker: &mut Checker, expr: &Expr) {
+pub(crate) fn unnecessary_list_index_lookup_comprehension(checker: &Checker, expr: &Expr) {
     let (Expr::Generator(ast::ExprGenerator {
         elt, generators, ..
     })
@@ -105,12 +105,11 @@ pub(crate) fn unnecessary_list_index_lookup_comprehension(checker: &mut Checker,
         };
 
         for range in ranges {
-            let mut diagnostic = Diagnostic::new(UnnecessaryListIndexLookup, range);
+            let mut diagnostic = checker.report_diagnostic(UnnecessaryListIndexLookup, range);
             diagnostic.set_fix(Fix::safe_edits(
                 Edit::range_replacement(value_name.id.to_string(), range),
                 [noop(index_name), noop(value_name)],
             ));
-            checker.diagnostics.push(diagnostic);
         }
     }
 }
@@ -151,6 +150,22 @@ fn enumerate_items<'a>(
         return None;
     };
 
+    // If the `enumerate` call has a non-zero `start`, don't omit.
+    if !arguments
+        .find_argument_value("start", 1)
+        .is_none_or(|expr| {
+            matches!(
+                expr,
+                Expr::NumberLiteral(ast::ExprNumberLiteral {
+                    value: Number::Int(Int::ZERO),
+                    ..
+                })
+            )
+        })
+    {
+        return None;
+    }
+
     // Check that the function is the `enumerate` builtin.
     if !semantic.match_builtin_expr(func, "enumerate") {
         return None;
