@@ -3,9 +3,8 @@ use ruff_python_ast::AnyNodeRef;
 use ruff_python_ast::ExprList;
 use ruff_text_size::Ranged;
 
-use crate::comments::SourceComment;
 use crate::expression::parentheses::{
-    empty_parenthesized, parenthesized, NeedsParentheses, OptionalParentheses,
+    NeedsParentheses, OptionalParentheses, empty_parenthesized, parenthesized,
 };
 use crate::prelude::*;
 
@@ -16,6 +15,7 @@ impl FormatNodeRule<ExprList> for FormatExprList {
     fn fmt_fields(&self, item: &ExprList, f: &mut PyFormatter) -> FormatResult<()> {
         let ExprList {
             range: _,
+            node_index: _,
             elts,
             ctx: _,
         } = item;
@@ -37,15 +37,6 @@ impl FormatNodeRule<ExprList> for FormatExprList {
             .with_dangling_comments(dangling)
             .fmt(f)
     }
-
-    fn fmt_dangling_comments(
-        &self,
-        _dangling_comments: &[SourceComment],
-        _f: &mut PyFormatter,
-    ) -> FormatResult<()> {
-        // Handled as part of `fmt_fields`
-        Ok(())
-    }
 }
 
 impl NeedsParentheses for ExprList {
