@@ -27,13 +27,15 @@
 //! [`await_request`]: TestServer::await_request
 //! [`await_notification`]: TestServer::await_notification
 
+mod code_actions;
+mod commands;
 mod initialize;
 mod inlay_hints;
+mod notebook;
 mod publish_diagnostics;
 mod pull_diagnostics;
 
-use std::collections::hash_map::Entry;
-use std::collections::{HashMap, VecDeque};
+use std::collections::{BTreeMap, HashMap, VecDeque};
 use std::num::NonZeroUsize;
 use std::sync::{Arc, OnceLock};
 use std::thread::JoinHandle;
@@ -86,32 +88,69 @@ fn setup_tracing() {
     });
 }
 
-/// Errors that can occur during testing
+/// Errors when receiving a notification or request from the server.
 #[derive(thiserror::Error, Debug)]
-pub(crate) enum TestServerError {
-    /// The response came back, but was an error response, not a successful one.
-    #[error("Response error: {0:?}")]
-    ResponseError(ResponseError),
+pub(crate) enum ServerMessageError {
+    #[error("waiting for message timed out")]
+    Timeout,
 
-    #[error("Invalid response message for request {0}: {1:?}")]
-    InvalidResponse(RequestId, Box<Response>),
+    #[error("server disconnected")]
+    ServerDisconnected,
 
-    #[error("Got a duplicate response for request ID {0}: {1:?}")]
-    DuplicateResponse(RequestId, Box<Response>),
-
-    #[error("Failed to receive message from server: {0}")]
-    RecvTimeoutError(RecvTimeoutError),
+    #[error("Failed to deserialize message body: {0}")]
+    DeserializationError(#[from] serde_json::Error),
 }
 
-impl TestServerError {
-    fn is_disconnected(&self) -> bool {
-        matches!(
-            self,
-            TestServerError::RecvTimeoutError(RecvTimeoutError::Disconnected)
-        )
+impl From<ReceiveError> for ServerMessageError {
+    fn from(value: ReceiveError) -> Self {
+        match value {
+            ReceiveError::Timeout => Self::Timeout,
+            ReceiveError::ServerDisconnected => Self::ServerDisconnected,
+        }
     }
 }
 
+/// Errors when receiving a response from the server.
+#[derive(thiserror::Error, Debug)]
+pub(crate) enum AwaitResponseError {
+    /// The response came back, but was an error response, not a successful one.
+    #[error("request failed because the server replied with an error: {0:?}")]
+    RequestFailed(ResponseError),
+
+    #[error("malformed response message with both result and error: {0:#?}")]
+    MalformedResponse(Box<Response>),
+
+    #[error("received multiple responses for the same request ID: {0:#?}")]
+    MultipleResponses(Box<[Response]>),
+
+    #[error("waiting for response timed out")]
+    Timeout,
+
+    #[error("server disconnected")]
+    ServerDisconnected,
+
+    #[error("failed to deserialize response result: {0}")]
+    DeserializationError(#[from] serde_json::Error),
+}
+
+impl From<ReceiveError> for AwaitResponseError {
+    fn from(err: ReceiveError) -> Self {
+        match err {
+            ReceiveError::Timeout => Self::Timeout,
+            ReceiveError::ServerDisconnected => Self::ServerDisconnected,
+        }
+    }
+}
+
+#[derive(thiserror::Error, Debug)]
+pub(crate) enum ReceiveError {
+    #[error("waiting for message timed out")]
+    Timeout,
+
+    #[error("server disconnected")]
+    ServerDisconnected,
+}
+
 /// A test server for the ty language server that provides helpers for sending requests,
 /// correlating responses, and handling notifications.
 pub(crate) struct TestServer {
@@ -135,8 +174,12 @@ pub(crate) struct TestServer {
     /// Incrementing counter to automatically generate request IDs
     request_counter: i32,
 
-    /// A mapping of request IDs to responses received from the server
-    responses: FxHashMap<RequestId, Response>,
+    /// A mapping of request IDs to responses received from the server.
+    ///
+    /// Valid responses contain exactly one response but may contain multiple responses
+    /// when the server sends multiple responses for a single request.
+    /// The responses are guaranteed to never be empty.
+    responses: FxHashMap<RequestId, smallvec::SmallVec<[Response; 1]>>,
 
     /// An ordered queue of all the notifications received from the server
     notifications: VecDeque<lsp_server::Notification>,
@@ -162,9 +205,11 @@ impl TestServer {
         test_context: TestContext,
         capabilities: ClientCapabilities,
         initialization_options: Option<ClientOptions>,
-    ) -> Result<Self> {
+    ) -> Self {
         setup_tracing();
 
+        tracing::debug!("Starting test client with capabilities {:#?}", capabilities);
+
         let (server_connection, client_connection) = Connection::memory();
 
         // Create OS system with the test directory as cwd
@@ -223,7 +268,7 @@ impl TestServer {
         workspace_folders: Vec<WorkspaceFolder>,
         capabilities: ClientCapabilities,
         initialization_options: Option<ClientOptions>,
-    ) -> Result<Self> {
+    ) -> Self {
         let init_params = InitializeParams {
             capabilities,
             workspace_folders: Some(workspace_folders),
@@ -236,10 +281,10 @@ impl TestServer {
         };
 
         let init_request_id = self.send_request::<Initialize>(init_params);
-        self.initialize_response = Some(self.await_response::<Initialize>(&init_request_id)?);
+        self.initialize_response = Some(self.await_response::<Initialize>(&init_request_id));
         self.send_notification::<Initialized>(InitializedParams {});
 
-        Ok(self)
+        self
     }
 
     /// Wait until the server has initialized all workspaces.
@@ -248,28 +293,18 @@ impl TestServer {
     /// server, and handles the request.
     ///
     /// This should only be called if the server is expected to send this request.
-    pub(crate) fn wait_until_workspaces_are_initialized(mut self) -> Result<Self> {
-        let (request_id, params) = self.await_request::<WorkspaceConfiguration>()?;
-        self.handle_workspace_configuration_request(request_id, &params)?;
-        Ok(self)
+    #[track_caller]
+    pub(crate) fn wait_until_workspaces_are_initialized(mut self) -> Self {
+        let (request_id, params) = self.await_request::<WorkspaceConfiguration>();
+        self.handle_workspace_configuration_request(request_id, &params);
+        self
     }
 
     /// Drain all messages from the server.
     fn drain_messages(&mut self) {
-        loop {
-            // Don't wait too long to drain the messages, as this is called in the `Drop`
-            // implementation which happens everytime the test ends.
-            match self.receive(Some(Duration::from_millis(10))) {
-                Ok(()) => {}
-                Err(TestServerError::RecvTimeoutError(_)) => {
-                    // Only break if we have no more messages to process.
-                    break;
-                }
-                Err(err) => {
-                    tracing::error!("Error while draining messages: {err:?}");
-                }
-            }
-        }
+        // Don't wait too long to drain the messages, as this is called in the `Drop`
+        // implementation which happens everytime the test ends.
+        while let Ok(()) = self.receive(Some(Duration::from_millis(10))) {}
     }
 
     /// Validate that there are no pending messages from the server.
@@ -344,6 +379,7 @@ impl TestServer {
         }
 
         let id = self.next_request_id();
+        tracing::debug!("Client sends request `{}` with ID {}", R::METHOD, id);
         let request = lsp_server::Request::new(id.clone(), R::METHOD.to_string(), params);
         self.send(Message::Request(request));
         id
@@ -355,6 +391,7 @@ impl TestServer {
         N: Notification,
     {
         let notification = lsp_server::Notification::new(N::METHOD.to_string(), params);
+        tracing::debug!("Client sends notification `{}`", N::METHOD);
         self.send(Message::Notification(notification));
     }
 
@@ -366,13 +403,48 @@ impl TestServer {
     /// This method will remove the response from the internal data structure, so it can only be
     /// called once per request ID.
     ///
+    /// # Panics
+    ///
+    /// If the server didn't send a response, the response failed with an error code, failed to deserialize,
+    /// or the server responded twice. Use [`Self::try_await_response`] if you want a non-panicking version.
+    ///
     /// [`send_request`]: TestServer::send_request
-    pub(crate) fn await_response<R>(&mut self, id: &RequestId) -> Result<R::Result>
+    #[track_caller]
+    pub(crate) fn await_response<R>(&mut self, id: &RequestId) -> R::Result
+    where
+        R: Request,
+    {
+        self.try_await_response::<R>(id, None)
+            .unwrap_or_else(|err| panic!("Failed to receive response for request {id}: {err}"))
+    }
+
+    /// Wait for a server response corresponding to the given request ID.
+    ///
+    /// This should only be called if a request was already sent to the server via [`send_request`]
+    /// which returns the request ID that should be used here.
+    ///
+    /// This method will remove the response from the internal data structure, so it can only be
+    /// called once per request ID.
+    ///
+    /// [`send_request`]: TestServer::send_request
+    pub(crate) fn try_await_response<R>(
+        &mut self,
+        id: &RequestId,
+        timeout: Option<Duration>,
+    ) -> Result<R::Result, AwaitResponseError>
     where
         R: Request,
     {
         loop {
-            if let Some(response) = self.responses.remove(id) {
+            if let Some(mut responses) = self.responses.remove(id) {
+                if responses.len() > 1 {
+                    return Err(AwaitResponseError::MultipleResponses(
+                        responses.into_boxed_slice(),
+                    ));
+                }
+
+                let response = responses.pop().unwrap();
+
                 match response {
                     Response {
                         error: None,
@@ -386,19 +458,15 @@ impl TestServer {
                         result: None,
                         ..
                     } => {
-                        return Err(TestServerError::ResponseError(err).into());
+                        return Err(AwaitResponseError::RequestFailed(err));
                     }
                     response => {
-                        return Err(TestServerError::InvalidResponse(
-                            id.clone(),
-                            Box::new(response),
-                        )
-                        .into());
+                        return Err(AwaitResponseError::MalformedResponse(Box::new(response)));
                     }
                 }
             }
 
-            self.receive_or_panic()?;
+            self.receive(timeout)?;
         }
     }
 
@@ -411,7 +479,31 @@ impl TestServer {
     ///
     /// This method will remove the notification from the internal data structure, so it should
     /// only be called if the notification is expected to be sent by the server.
-    pub(crate) fn await_notification<N: Notification>(&mut self) -> Result<N::Params> {
+    ///
+    /// # Panics
+    ///
+    /// If the server doesn't send the notification within the default timeout or
+    /// the notification failed to deserialize. Use [`Self::try_await_notification`] for
+    /// a panic-free alternative.
+    #[track_caller]
+    pub(crate) fn await_notification<N: Notification>(&mut self) -> N::Params {
+        self.try_await_notification::<N>(None)
+            .unwrap_or_else(|err| panic!("Failed to receive notification `{}`: {err}", N::METHOD))
+    }
+
+    /// Wait for a notification of the specified type from the server and return its parameters.
+    ///
+    /// The caller should ensure that the server is expected to send this notification type. It
+    /// will keep polling the server for this notification up to 10 times before giving up after
+    /// which it will return an error. It will also return an error if the notification is not
+    /// received within `recv_timeout` duration.
+    ///
+    /// This method will remove the notification from the internal data structure, so it should
+    /// only be called if the notification is expected to be sent by the server.
+    pub(crate) fn try_await_notification<N: Notification>(
+        &mut self,
+        timeout: Option<Duration>,
+    ) -> Result<N::Params, ServerMessageError> {
         for retry_count in 0..RETRY_COUNT {
             if retry_count > 0 {
                 tracing::info!("Retrying to receive `{}` notification", N::METHOD);
@@ -422,14 +514,42 @@ impl TestServer {
                 .position(|notification| N::METHOD == notification.method)
                 .and_then(|index| self.notifications.remove(index));
             if let Some(notification) = notification {
-                return Ok(serde_json::from_value(notification.params)?);
+                let params = serde_json::from_value(notification.params)?;
+                return Ok(params);
             }
-            self.receive_or_panic()?;
+
+            self.receive(timeout)?;
         }
-        Err(anyhow::anyhow!(
-            "Failed to receive `{}` notification after {RETRY_COUNT} retries",
-            N::METHOD
-        ))
+
+        Err(ServerMessageError::Timeout)
+    }
+
+    /// Collects `N` publish diagnostic notifications into a map, indexed by the document url.
+    ///
+    /// ## Panics
+    /// If there are multiple publish diagnostics notifications for the same document.
+    #[track_caller]
+    pub(crate) fn collect_publish_diagnostic_notifications(
+        &mut self,
+        count: usize,
+    ) -> BTreeMap<lsp_types::Url, Vec<lsp_types::Diagnostic>> {
+        let mut results = BTreeMap::default();
+
+        for _ in 0..count {
+            let notification =
+                self.await_notification::<lsp_types::notification::PublishDiagnostics>();
+
+            if let Some(existing) =
+                results.insert(notification.uri.clone(), notification.diagnostics)
+            {
+                panic!(
+                    "Received multiple publish diagnostic notifications for {url}: ({existing:#?})",
+                    url = &notification.uri
+                );
+            }
+        }
+
+        results
     }
 
     /// Wait for a request of the specified type from the server and return the request ID and
@@ -442,7 +562,31 @@ impl TestServer {
     ///
     /// This method will remove the request from the internal data structure, so it should only be
     /// called if the request is expected to be sent by the server.
-    pub(crate) fn await_request<R: Request>(&mut self) -> Result<(RequestId, R::Params)> {
+    ///
+    /// # Panics
+    ///
+    /// If receiving the request fails.
+    #[track_caller]
+    pub(crate) fn await_request<R: Request>(&mut self) -> (RequestId, R::Params) {
+        self.try_await_request::<R>(None)
+            .unwrap_or_else(|err| panic!("Failed to receive server request `{}`: {err}", R::METHOD))
+    }
+
+    /// Wait for a request of the specified type from the server and return the request ID and
+    /// parameters.
+    ///
+    /// The caller should ensure that the server is expected to send this request type. It will
+    /// keep polling the server for this request up to 10 times before giving up after which it
+    /// will return an error. It can also return an error if the request is not received within
+    /// `recv_timeout` duration.
+    ///
+    /// This method will remove the request from the internal data structure, so it should only be
+    /// called if the request is expected to be sent by the server.
+    #[track_caller]
+    pub(crate) fn try_await_request<R: Request>(
+        &mut self,
+        timeout: Option<Duration>,
+    ) -> Result<(RequestId, R::Params), ServerMessageError> {
         for retry_count in 0..RETRY_COUNT {
             if retry_count > 0 {
                 tracing::info!("Retrying to receive `{}` request", R::METHOD);
@@ -456,12 +600,10 @@ impl TestServer {
                 let params = serde_json::from_value(request.params)?;
                 return Ok((request.id, params));
             }
-            self.receive_or_panic()?;
+
+            self.receive(timeout)?;
         }
-        Err(anyhow::anyhow!(
-            "Failed to receive `{}` request after {RETRY_COUNT} retries",
-            R::METHOD
-        ))
+        Err(ServerMessageError::Timeout)
     }
 
     /// Receive a message from the server.
@@ -469,71 +611,51 @@ impl TestServer {
     /// It will wait for `timeout` duration for a message to arrive. If no message is received
     /// within that time, it will return an error.
     ///
-    /// If `timeout` is `None`, it will use a default timeout of 1 second.
-    fn receive(&mut self, timeout: Option<Duration>) -> Result<(), TestServerError> {
-        static DEFAULT_TIMEOUT: Duration = Duration::from_secs(1);
+    /// If `timeout` is `None`, it will use a default timeout of 10 second.
+    fn receive(&mut self, timeout: Option<Duration>) -> Result<(), ReceiveError> {
+        static DEFAULT_TIMEOUT: Duration = Duration::from_secs(10);
 
         let receiver = self.client_connection.as_ref().unwrap().receiver.clone();
         let message = receiver
             .recv_timeout(timeout.unwrap_or(DEFAULT_TIMEOUT))
-            .map_err(TestServerError::RecvTimeoutError)?;
+            .map_err(|err| match err {
+                RecvTimeoutError::Disconnected => ReceiveError::ServerDisconnected,
+                RecvTimeoutError::Timeout => ReceiveError::Timeout,
+            })?;
 
-        self.handle_message(message)?;
+        self.handle_message(message);
 
         for message in receiver.try_iter() {
-            self.handle_message(message)?;
+            self.handle_message(message);
         }
 
         Ok(())
     }
 
-    /// This is a convenience method that's same as [`receive`], but panics if the server got
-    /// disconnected. It will pass other errors as is.
-    ///
-    /// [`receive`]: TestServer::receive
-    fn receive_or_panic(&mut self) -> Result<(), TestServerError> {
-        if let Err(err) = self.receive(None) {
-            if err.is_disconnected() {
-                self.panic_on_server_disconnect();
-            } else {
-                return Err(err);
-            }
-        }
-        Ok(())
-    }
-
     /// Handle the incoming message from the server.
     ///
     /// This method will store the message as follows:
     /// - Requests are stored in `self.requests`
     /// - Responses are stored in `self.responses` with the request ID as the key
     /// - Notifications are stored in `self.notifications`
-    fn handle_message(&mut self, message: Message) -> Result<(), TestServerError> {
+    fn handle_message(&mut self, message: Message) {
         match message {
             Message::Request(request) => {
-                tracing::debug!("Received server request {}", &request.method);
+                tracing::debug!("Received server request `{}`", &request.method);
                 self.requests.push_back(request);
             }
             Message::Response(response) => {
                 tracing::debug!("Received server response for request {}", &response.id);
-                match self.responses.entry(response.id.clone()) {
-                    Entry::Occupied(existing) => {
-                        return Err(TestServerError::DuplicateResponse(
-                            response.id,
-                            Box::new(existing.get().clone()),
-                        ));
-                    }
-                    Entry::Vacant(entry) => {
-                        entry.insert(response);
-                    }
-                }
+                self.responses
+                    .entry(response.id.clone())
+                    .or_default()
+                    .push(response);
             }
             Message::Notification(notification) => {
-                tracing::debug!("Received notification {}", &notification.method);
+                tracing::debug!("Received notification `{}`", &notification.method);
                 self.notifications.push_back(notification);
             }
         }
-        Ok(())
     }
 
     #[track_caller]
@@ -566,11 +688,12 @@ impl TestServer {
     /// Use the [`get_request`] method to wait for the server to send this request.
     ///
     /// [`get_request`]: TestServer::get_request
+    #[track_caller]
     fn handle_workspace_configuration_request(
         &mut self,
         request_id: RequestId,
         params: &ConfigurationParams,
-    ) -> Result<()> {
+    ) {
         let mut results = Vec::new();
 
         for item in &params.items {
@@ -585,7 +708,12 @@ impl TestServer {
                 // > If the client can't provide a configuration setting for a given scope
                 // > then null needs to be present in the returned array.
                 match item.section.as_deref() {
-                    Some("ty") => serde_json::to_value(options)?,
+                    Some("ty") => match serde_json::to_value(options) {
+                        Ok(value) => value,
+                        Err(err) => {
+                            panic!("Failed to deserialize workspace configuration options: {err}",)
+                        }
+                    },
                     Some(section) => {
                         tracing::debug!("Unrecognized section `{section}` for {scope_uri}");
                         serde_json::Value::Null
@@ -606,8 +734,6 @@ impl TestServer {
 
         let response = Response::new_ok(request_id, results);
         self.send(Message::Response(response));
-
-        Ok(())
     }
 
     /// Get the initialization result
@@ -615,16 +741,19 @@ impl TestServer {
         self.initialize_response.as_ref()
     }
 
-    fn file_uri(&self, path: impl AsRef<SystemPath>) -> Url {
-        Url::from_file_path(self.test_context.root().join(path.as_ref()).as_std_path())
-            .expect("Path must be a valid URL")
+    pub(crate) fn file_uri(&self, path: impl AsRef<SystemPath>) -> Url {
+        Url::from_file_path(self.file_path(path).as_std_path()).expect("Path must be a valid URL")
+    }
+
+    pub(crate) fn file_path(&self, path: impl AsRef<SystemPath>) -> SystemPathBuf {
+        self.test_context.root().join(path)
     }
 
     /// Send a `textDocument/didOpen` notification
     pub(crate) fn open_text_document(
         &mut self,
         path: impl AsRef<SystemPath>,
-        content: &impl ToString,
+        content: impl AsRef<str>,
         version: i32,
     ) {
         let params = DidOpenTextDocumentParams {
@@ -632,7 +761,7 @@ impl TestServer {
                 uri: self.file_uri(path),
                 language_id: "python".to_string(),
                 version,
-                text: content.to_string(),
+                text: content.as_ref().to_string(),
             },
         };
         self.send_notification::<DidOpenTextDocument>(params);
@@ -667,7 +796,6 @@ impl TestServer {
     }
 
     /// Send a `workspace/didChangeWatchedFiles` notification with the given file events
-    #[expect(dead_code)]
     pub(crate) fn did_change_watched_files(&mut self, events: Vec<FileEvent>) {
         let params = DidChangeWatchedFilesParams { changes: events };
         self.send_notification::<DidChangeWatchedFiles>(params);
@@ -678,7 +806,7 @@ impl TestServer {
         &mut self,
         path: impl AsRef<SystemPath>,
         previous_result_id: Option<String>,
-    ) -> Result<DocumentDiagnosticReportResult> {
+    ) -> DocumentDiagnosticReportResult {
         let params = DocumentDiagnosticParams {
             text_document: TextDocumentIdentifier {
                 uri: self.file_uri(path),
@@ -697,7 +825,7 @@ impl TestServer {
         &mut self,
         work_done_token: Option<lsp_types::NumberOrString>,
         previous_result_ids: Option<Vec<PreviousResultId>>,
-    ) -> Result<WorkspaceDiagnosticReportResult> {
+    ) -> WorkspaceDiagnosticReportResult {
         let params = WorkspaceDiagnosticParams {
             identifier: Some("ty".to_string()),
             previous_result_ids: previous_result_ids.unwrap_or_default(),
@@ -714,7 +842,7 @@ impl TestServer {
         &mut self,
         path: impl AsRef<SystemPath>,
         position: Position,
-    ) -> Result<Option<Hover>> {
+    ) -> Option<Hover> {
         let params = HoverParams {
             text_document_position_params: TextDocumentPositionParams {
                 text_document: TextDocumentIdentifier {
@@ -733,7 +861,7 @@ impl TestServer {
         &mut self,
         path: impl AsRef<SystemPath>,
         range: Range,
-    ) -> Result<Option<Vec<InlayHint>>> {
+    ) -> Option<Vec<InlayHint>> {
         let params = InlayHintParams {
             text_document: TextDocumentIdentifier {
                 uri: self.file_uri(path),
@@ -770,9 +898,10 @@ impl Drop for TestServer {
         // it dropped the client connection.
         let shutdown_error = if self.server_thread.is_some() && !self.shutdown_requested {
             let shutdown_id = self.send_request::<Shutdown>(());
-            match self.await_response::<Shutdown>(&shutdown_id) {
+            match self.try_await_response::<Shutdown>(&shutdown_id, None) {
                 Ok(()) => {
                     self.send_notification::<Exit>(());
+
                     None
                 }
                 Err(err) => Some(format!("Failed to get shutdown response: {err:?}")),
@@ -781,9 +910,29 @@ impl Drop for TestServer {
             None
         };
 
-        if let Some(_client_connection) = self.client_connection.take() {
-            // Drop the client connection before joining the server thread to avoid any hangs
-            // in case the server didn't respond to the shutdown request.
+        // Drop the client connection before joining the server thread to avoid any hangs
+        // in case the server didn't respond to the shutdown request.
+        if let Some(client_connection) = self.client_connection.take() {
+            if !std::thread::panicking() {
+                // Wait for the client sender to drop (confirmation that it processed the exit notification).
+
+                match client_connection
+                    .receiver
+                    .recv_timeout(Duration::from_secs(20))
+                {
+                    Err(RecvTimeoutError::Disconnected) => {
+                        // Good, the server terminated
+                    }
+                    Err(RecvTimeoutError::Timeout) => {
+                        tracing::warn!(
+                            "The server didn't exit within 20ms after receiving the EXIT notification"
+                        );
+                    }
+                    Ok(message) => {
+                        self.handle_message(message);
+                    }
+                }
+            }
         }
 
         if std::thread::panicking() {
@@ -991,7 +1140,7 @@ impl TestServerBuilder {
     }
 
     /// Build the test server
-    pub(crate) fn build(self) -> Result<TestServer> {
+    pub(crate) fn build(self) -> TestServer {
         TestServer::new(
             self.workspaces,
             self.test_context,
@@ -1052,6 +1201,7 @@ impl TestContext {
             r#"The system cannot find the file specified."#,
             "No such file or directory",
         );
+        settings.add_filter(r"file://.*/stdlib/", "file://<typeshed>/stdlib/");
 
         let settings_scope = settings.bind_to_scope();
 
